#include "hardware/springprox/springprox.h"

#include <stdio.h>
#include <string.h>

#include "fingertk.h"
#include "sagem_mso_ilv.h"

BOOL    enroll_ex(FINGERTK_DATA_ST * data);
void    enroll_async_action(BYTE code);
void    enroll_async_info(BYTE finger_no, BYTE finger_count, BYTE capture_no, BYTE capture_count);

/* Perform enroll */
/* -------------- */
BOOL enroll(FINGERTK_DATA_ST * data)
{
  char    temp[MAX_PATH];

  printf("Enrolling a new user\n");

  memset(data, 0, sizeof(FINGERTK_DATA_ST));

  /* Get user's ID */
  printf("Enter user's ID: ");
  memset(temp, 0, sizeof(temp));
  if ((gets(temp) == NULL) || (strlen(temp) == 0))
  {
    printf("Cancelled\n");
    return FALSE;
  }
  strncpy(data->id, temp, sizeof(data->id));

  /* Get user's name */
  printf("Enter user's lastname: ");
  memset(temp, 0, sizeof(temp));
  if ((gets(temp) == NULL) || (strlen(temp) == 0))
  {
    printf("Cancelled\n");
    return FALSE;
  }
  strncpy(data->lastname, temp, sizeof(data->lastname));

  printf("Enter user's firstname: ");
  memset(temp, 0, sizeof(temp));
  if ((gets(temp) == NULL) || (strlen(temp) == 0))
  {
    printf("Cancelled\n");
    return FALSE;
  }
  strncpy(data->firstname, temp, sizeof(data->firstname));

  printf("\n");

  if (!enroll_ex(data))
  {
    printf("Enrollment failed\n");
    return FALSE;
  }

  printf("Enrollment done\n");
  return TRUE;
}

/* SAGEM MSO enrollment command (2 fingers) */
/* ---------------------------------------- */
BOOL enroll_ex(FINGERTK_DATA_ST * data)
{
  BYTE    buffer[1024];
  BYTE    async;
  WORD    size;
  BYTE    tag;
  WORD    offset, length;
  BYTE   *value;
  BYTE    idx;
  SWORD   rc;

  /* Build the ILV enroll command               */
  /* (SEE SAGEM DOCUMENTATION FOR EXPLANATIONS) */
  /* ------------------------------------------ */

  memset(buffer, 0, sizeof(buffer));
  length = 0;

  /* Initialize ILV */
  ilv_init(buffer, &length, ILV_ENROLL);

  /* Build ILV */
  ilv_add_byte(buffer, &length, 0); /* Database identifier = 0 */
  ilv_add_word(buffer, &length, 60);  /* Timeout = 60 seconds */
  ilv_add_byte(buffer, &length, 0); /* Acquisition quality threshold = default (0) */
  ilv_add_byte(buffer, &length, 0); /* Enrollment type = default (0) */
  ilv_add_byte(buffer, &length, 2); /* We want to enroll 2 fingers */
  ilv_add_byte(buffer, &length, 0); /* Save record into local database = NO (0) */
  ilv_add_byte(buffer, &length, 1); /* Export minutiae with its default size = YES (1) */

  /* We want to receive asynchronous events */
  ilv_add_byte(buffer, &length, ID_ASYNCHRONOUS_EVENT);
  ilv_add_word(buffer, &length, 4);
  ilv_add_dword(buffer, &length, 0x00000005);
  /* Bit field = 0x00000005 : action messages (move left, press harder...) */
  /* and information messages (finger X/X, capture X/X) will be sent       */

  /* Write length in the ILV */
  ilv_final(buffer, &length);

  /* Send the command to the MSO and process its responses (including asynchronous messages) */
  /* --------------------------------------------------------------------------------------- */
  async = FALSE;
  for (;;)
  {
    size = sizeof(buffer);

    /* NOTE : the timeout of the exchange must be a little greater than the timeout offered  */
    /*        to the MSO, overwhise the library will return an error code before the MSO has */
    /*        finished is process                                                            */
    if (async)
      rc = SPROX_MSO_Exchange(NULL, 0, buffer, &size, 65000, &async);
    else
      rc = SPROX_MSO_Exchange(buffer, length, buffer, &size, 65000, &async);

    if (rc != 0)
      return FALSE;

    if (async)
    {
      /* Easy processing of asynchronous message, no error control */
      switch (buffer[4])
      {
        case 0x01:
          enroll_async_action(buffer[7]);
          break;
        case 0x04:
          enroll_async_info(buffer[7], buffer[8], buffer[9], buffer[10]);
          break;
      }

      /* NOTE : we MUST go back to MSO_Exchange AS SOON AS POSSIBLE because if we were to  */
      /*        miss any message (asynchronous or final result), the reader will report a  */
      /*        protocol error and abort the process                                       */
    } else
    {
      break;                    /* Last message -> end of the loop */
    }
  }

  /* 
   * Reply format is :
   *  0      : reply ID
   *  1 -> 2 : reply length
   *  3      : request status
   *  4      : enroll status
   *  5 -> 8 : user database index
   *  9      : first finger
   *  ...    : second finger
   */

  /* Check returned reply and status */
  /* ------------------------------- */

  if (buffer[0] != ILV_ENROLL)
  {
    printf("Enroll failed (MSO reply=0x%02X)\n", buffer[0]);
    return FALSE;
  }
  if (buffer[3] != 0x00)
  {
    printf("Enroll failed (MSO request status=0x%02X)\n", buffer[3]);
    return FALSE;
  }
  if (buffer[4] != 0x00)
  {
    printf("Enroll failed (MSO enroll status=0x%02X)\n", buffer[4]);
    return FALSE;
  }

  /* Decode returned data */
  /* -------------------- */

  offset = 9;
  idx = 0;
  while ((offset < size) && ilv_loop(buffer, &offset, &tag, &length, &value))
  {
    printf("Idx %d : tag %02X, length = %d\n", idx, tag, length);

    if ((tag != ID_PKCOMP)      /* PKCOMP minutiae */
        && (tag != ID_PKMAT))   /* PKMAT minutiae */
    {
      /* Skip */
      continue;
    }

    if (idx < 2)
    {
      /* Finger 1 or 2 */
      if ((length > sizeof(data->finger1_data)) || (length > sizeof(data->finger2_data)))
      {
        printf("\tReceived fingerprint is to big !\n");
        continue;
      }
      if (idx == 0)
      {
        data->finger1_type = tag;
        memcpy(data->finger1_data, value, length);
        data->finger1_len = length;
      } else
      {
        data->finger2_type = tag;
        memcpy(data->finger2_data, value, length);
        data->finger2_len = length;
      }
      idx++;
    } else
    {
      /* Silently ignore the finger */
    }
  }

  return TRUE;
}

void enroll_async_action(BYTE code)
{
  printf("\t\t");

  switch (code)
  {
    case 0:
      /* printf("Place finger"); */
      break;
    case 1:
      printf("Move up");
      break;
    case 2:
      printf("Move down");
      break;
    case 3:
      printf("Move left");
      break;
    case 4:
      printf("Move right");
      break;
    case 5:
      printf("Press harder");
      break;
    case 6:
      printf("Move finger");
      break;
    case 7:
      printf("Remove finger");
      break;
    case 8:
      printf("Finger OK");
      break;
  }

  printf("\n");
}

void enroll_async_info(BYTE finger_no, BYTE finger_count, BYTE capture_no, BYTE capture_count)
{
  printf("\tFinger %d/%d, capture %d/%d\n", finger_no, finger_count, capture_no, capture_count);
}
