#include "sagem_mso_i.h"

/*
 * ILV decoder
 * -----------
 */

/* Function to loop inside an ILV list */
/* ----------------------------------- */
BOOL MSO_ILV_Loop(BYTE *buffer, WORD *offset, BYTE *tag, WORD *length, BYTE **value)
{
  WORD    l;

  if ((buffer == NULL) || (offset == NULL))
    return FALSE;

  if (buffer[*offset] == 0x00)
    return FALSE;

  if (tag != NULL)
    *tag = buffer[*offset];
  l = buffer[*offset + 2];
  l *= 0x0100;
  l += buffer[*offset + 1];
  if (length != NULL)
    *length = l;
  if (value != NULL)
    *value = &buffer[*offset + 3];

  *offset = *offset + 3 + l;

  if (l > 1024)
    return FALSE;

  return TRUE;
}

/*
 * ILV encoder
 * -----------
 */

/* Start the building the ILV buffer */
void MSO_ILV_Init(BYTE *buffer, WORD *size, BYTE tag)
{
  /* Command appears on top of the buffer */
  buffer[0] = tag;

  /* Actual length will be written at last */
  buffer[1] = 0;
  buffer[2] = 0;

  /* OK */
  *size = 3;
}

/* Done with ILV buffer */
void MSO_ILV_Final(BYTE *buffer, WORD *size)
{
  WORD    length;

  /* Compute length of the VALUE field */
  if (*size < 3)
  {
    /* Oups... */
    *size = 0;
    return;
  }

  length = *size - 3;

  /* Write actual length */
  buffer[1] = (BYTE) (length % 0x0100);
  buffer[2] = (BYTE) (length / 0x0100);
}

/* Add a DWORD value to the ILV buffer */
void MSO_ILV_AddDWord(BYTE *buffer, WORD *size, DWORD value)
{
  /* Enqueue the WORD (Little Endian) */
  buffer[*size + 0] = (BYTE) (value % 0x00000100);
  buffer[*size + 1] = (BYTE) (value / 0x00000100);
  buffer[*size + 2] = (BYTE) (value / 0x00010000);
  buffer[*size + 3] = (BYTE) (value / 0x01000000);

  /* OK */
  *size = *size + 4;
}

/* Add a WORD value to the ILV buffer */
void MSO_ILV_AddWord(BYTE *buffer, WORD *size, WORD value)
{
  /* Enqueue the WORD (Little Endian) */
  buffer[*size + 0] = (BYTE) (value % 0x0100);
  buffer[*size + 1] = (BYTE) (value / 0x0100);

  /* OK */
  *size = *size + 2;
}

/* Add a BYTE value to the ILV buffer */
void MSO_ILV_AddByte(BYTE *buffer, WORD *size, BYTE value)
{
  buffer[*size + 0] = value;
  *size = *size + 1;
}

/* Add a data buffer to the ILV buffer */
void MSO_ILV_AddData(BYTE *buffer, WORD *size, BYTE *data, WORD length)
{
  /* Enqueue the buffer */
  memcpy(&buffer[*size + 0], data, length);
  *size = *size + length;
}

/* Check whever this is an asynchronous message or a "regular" one */
BOOL MSO_IsAsync(BYTE *buffer)
{
  BYTE    tag;
  WORD    offset = 0;

  if (!MSO_ILV_Loop(buffer, &offset, &tag, NULL, NULL))
    return FALSE;

  if (tag == ILV_ASYNC_MESSAGE)
    return TRUE;

  return FALSE;
}
