#include "project.h"
#include "sagem_mso_i.h"

//#define _DEBUG_MSO

static  WORD msoBaudrate;
BYTE    msoRequestCounter = 0xFF;

void    MSO_CRCUpdate(WORD * crc, BYTE val);

/*
 * Send to MSO
 * -----------
 * ID and RC are provided by caller
 * Stuffing, CRC and frame markers are performed here
 * We also wait for an ACK (or NACK)
 */
void send_with_stuffing(BYTE b)
{
  if (b == XON)
  {
    serial_0_send_byte(ESC);
    serial_0_send_byte(XON + 1);
  } else if (b == XOFF)
  {
    serial_0_send_byte(ESC);
    serial_0_send_byte(XOFF + 1);
  } else if (b == ESC)
  {
    serial_0_send_byte(ESC);
    serial_0_send_byte(ESC);
  } else
  {
    serial_0_send_byte(b);
  }
}

MSO_STATUS MSO_PP_Send(BYTE msoID, BYTE msoRC, BYTE * msoData, WORD msoDataLength)
{
  WORD    crc;
  WORD    offset;
  MSO_STATUS rc;
  BYTE    nack_count = 0;
  BYTE    repeat_count = 0;

  sprox_select_mso(msoBaudrate);

  for (;;)
  {
    watchdog_update();

    memset((BYTE *) sprox_external.buffer, 0xCC, sizeof(sprox_external.buffer));
    sprox_external.offset = 0;
    crc = 0x0000;

#ifdef _DEBUG_MSO
    print_s("<S>");
    print_s("ID=");
    print_hb(msoID);
    print_s(" RC=");
    print_hb(msoRC);
    print_s(" ");
#endif

    serial_0_send_byte(STX);    /* STX */
    serial_0_send_byte(msoID);  /* ID */
    send_with_stuffing(msoRC);  /* RC */

    if ((msoData == NULL) || (msoDataLength == 0))
    {
      /* Not a packet data */
#ifdef _DEBUG_MSO
      print_s("</S>");
      print_crlf();
#endif
      return ILV_OK;
    }

    /* Send data and compute CRC on the fly */
    for (offset = 0; offset < msoDataLength; offset++)
    {
#ifdef _DEBUG_MSO
      print_hb(msoData[offset]);
//      PrintChr(msoData[offset]);
      watchdog_update();
#endif
      MSO_CRCUpdate(&crc, msoData[offset]);
      send_with_stuffing(msoData[offset]);

      if ((offset & 0x000F) == 0x000F)
        watchdog_update();
    }

    /* CRC is appened after data */
#ifdef _DEBUG_MSO
    print_s(" crc=");
    print_hb(crc % 0x0100);
    print_hb(crc / 0x0100);
#endif

    send_with_stuffing(crc % 0x0100);
    send_with_stuffing(crc / 0x0100);

#ifdef _DEBUG_MSO
    print_s("</S>");
    print_crlf();
#endif

    serial_0_send_byte(ESC);    /* ESC */
    serial_0_send_byte(ETX);    /* ETX */
    sprox_external.mso_state = MSO_STATE_WAIT_ACK;

    watchdog_update();

    /* Wait for the ACK */
    if (repeat_count == 0)
    {
      sprox_external.timeout = timeout_init(500);
    } else
    {
      sprox_external.timeout = timeout_init(1000);
    }

    rc = ILV_OK;
    while (sprox_external.mso_state == MSO_STATE_WAIT_ACK)
    {
      if (timeout_expired(sprox_external.timeout))
      {
#ifdef _DEBUG_MSO
        print_s("<TMO>");
#endif
        rc = ILVERR_NO_RESPONSE;
        break;
      }
      watchdog_update();
    }

    sprox_external.timeout = timeout_kill();

    if (rc != ILV_OK)
    {
      /* Timeout ? We may retry */
      repeat_count++;
      if (repeat_count >= 3)
        return rc;
      continue;
    }

    if (sprox_external.mso_state == MSO_STATE_GOT_NACK)
    {
      /* NACK has been found */
      nack_count++;
      if (nack_count >= 5)
        return ILVERR_PROTOCOL;
      continue;
    }

    if ((sprox_external.mso_state == MSO_STATE_WAIT_DATA) || (sprox_external.mso_state == MSO_STATE_GOT_DATA))
    {
      /* ACK has been found */
      break;
    }

    /* Default : wrong receive sequence */
    return ILVERR_PROTOCOL;
  }

  return ILV_OK;
}

/*
 * Recv to MSO
 * -----------
 * ID and RC are forwarded by caller
 * Stuffing, CRC and frame markers are removed (and checked) here
 */
MSO_STATUS MSO_PP_Recv(BYTE *msoID, BYTE *msoRC, BYTE *msoData, WORD *msoDataLength, DWORD timeout)
{
  MSO_STATUS rc;
  WORD    i, j;
  WORD    recv_length;
  WORD    crc;

#ifdef _DEBUG_MSO
  print_s("<R>");
#endif

  rc = ILV_OK;

  /* Receive until end of frame or timeout */
  /* ------------------------------------- */

  /* We send a "more time" frame to the host every 1s */
  if (timeout == 0)
  {
    /* Infinite timeout */
    while ((sprox_external.mso_state == MSO_STATE_WAIT_DATA) && (!sprox_interrupted))
    {
      sprox_external.timeout = timeout_init(1000);
      watchdog_update();

      while ((sprox_external.mso_state == MSO_STATE_WAIT_DATA) && (!sprox_interrupted))
      {
        if (timeout_expired(sprox_external.timeout))
        {
          sprox_time_extension();  /* Tell host we're still working */
          break;
        }
        watchdog_update();
      }
    }
  } else
  while ((sprox_external.mso_state == MSO_STATE_WAIT_DATA) && (timeout > 0) && (!sprox_interrupted))
  {
    /* Finite timeout */
    if (timeout > 1000)
    {
      sprox_external.timeout = timeout_init(1000);
      timeout -= 1000;
    } else
    if (timeout > 0)
    {
      sprox_external.timeout = timeout_init(timeout);
      timeout = 0;
    }

    watchdog_update();

    while ((sprox_external.mso_state == MSO_STATE_WAIT_DATA) && (!sprox_interrupted))
    {
      watchdog_update();
      if (timeout_expired(sprox_external.timeout))
      {
#ifdef _DEBUG_MSO
        print_b('.');
#endif
        if (timeout > 0)      /* We still have some time */
          sprox_time_extension();  /* Tell host we're still working */
        else
          rc = ILVERR_NO_RESPONSE;  /* Exit the loop */
        break;
      }
    }
  }

  watchdog_update();
  sprox_external.timeout = timeout_kill();

  if (sprox_interrupted)
    rc = ILVERR_INTERRUPTED;

  if (rc == ILV_OK)
  {
    if (sprox_external.mso_state != MSO_STATE_GOT_DATA)
    {
      rc = ILVERR_PROTOCOL;
    } else
    if (sprox_external.buffer[0] != STX)
    {
      rc = ILVERR_FRAME;
    }
  }
  
  /* Remember received length */
  recv_length = sprox_external.offset;
  /* Prevent further receive during processing */
  sprox_external.offset = sizeof(sprox_external.buffer);

  /* Remove stuffing (i loops in src, j loops in dst) */
  j = 2;
  for (i = 2; i < recv_length; i++)
  {
    if (sprox_external.buffer[i] == ESC)
    {
      switch (sprox_external.buffer[i + 1])
      {
        case XON + 1:
          sprox_external.buffer[j] = XON;
          break;
        case XOFF + 1:
          sprox_external.buffer[j] = XOFF;
          break;
        case ESC:
          sprox_external.buffer[j] = ESC;
          break;
        case ETX:
          recv_length = j;      /* Length is now the real length */
          sprox_external.buffer[j] = 0xDD;
          break;
        default:
          sprox_external.buffer[j] = 0xEE;
          if (rc == ILV_OK)
            rc = ILVERR_FRAME;
      }
      i++;
    } else
    {
      sprox_external.buffer[j] = sprox_external.buffer[i];
    }
    j++;
  }

  /* Retrieve frame header */
  if (msoID != NULL)
    *msoID = sprox_external.buffer[1];
  if (msoRC != NULL)
    *msoRC = sprox_external.buffer[2];

#ifdef _DEBUG_MSO
  print_s("ID=");
  print_hb(sprox_external.buffer[1]);
  print_s(" RC=");
  print_hb(sprox_external.buffer[2]);
  print_s(" ");
#endif

  if (rc == ILV_OK)
  {
    /* Actual length : length - 3 (header) - 2 (CRC) */
    if (recv_length <= 4)
      rc = ILVERR_FRAME;
  }

  if ((rc != ILV_OK) || (recv_length <= 5))
  {
    if (msoDataLength != NULL)
      *msoDataLength = 0;
    goto leave;
  }

  /* Check CRC */
  crc = 0x0000;
  for (i=3; i<recv_length-2; i++)
  {
    MSO_CRCUpdate(&crc, sprox_external.buffer[i]);
#ifdef _DEBUG_MSO
    print_hb(sprox_external.buffer[i]);
    watchdog_update();
#endif
  }

  if ((sprox_external.buffer[recv_length - 2] != crc % 0x0100) || (sprox_external.buffer[recv_length - 1] != crc / 0x0100))
  {
    rc = ILVERR_CHECKSUM;
#ifdef _DEBUG_MSO
    print_s("(CRC ");
    print_hb(crc % 0x0100);
    print_hb(crc / 0x0100);
    print_s("!=");
    print_hb(sprox_external.buffer[recv_length - 2]);
    print_hb(sprox_external.buffer[recv_length - 1]);
    print_s(")");
    watchdog_update();
#endif
  }

  /* Copy data */
  recv_length -= 5;
  if (msoDataLength != NULL)
  {
    if ((*msoDataLength != 0) && (*msoDataLength < recv_length))
    {
      recv_length = *msoDataLength;
      if (rc == ILV_OK)
        rc = ILVERR_OVERFLOW;
    }
    *msoDataLength = recv_length;
  }

  if (msoData != NULL)
    memcpy(msoData, (BYTE *) & sprox_external.buffer[3], recv_length);

leave:
  /* Enable receiver again */
  sprox_external.offset = 0;
  sprox_external.mso_state = MSO_STATE_WAIT_DATA;

  /* Send ACK in order to prevent re-emitting */
  MSO_PP_Send(0x62, sprox_external.buffer[2], NULL, 0);

#ifdef _DEBUG_MSO
  switch (rc)
  {
    case ILV_OK             : break;
    case ILVERR_NO_RESPONSE : print_s("<TMO>"); break;
    case ILVERR_INTERRUPTED : print_s("<ESC>"); break;
    case ILVERR_CHECKSUM    : print_s("<CRC>"); break;
    case ILVERR_OVERFLOW    : print_s("<OVF>"); break;
    default                 : print_s("<err. "); print_hb(rc); print_s(">");
  }
  print_s("</R>\r\n");  
#endif

  return rc;
}

MSO_STATUS MSO_Open(void)
{
  BYTE retry;
  BYTE send_get[] = { 0x05, 0x01, 0x00, 0x2F };

  PERIPHS_ON();
    
  if (sprox_external.active.p.mso)
    return ILV_OK;
                   
  msoBaudrate = 9600;
  sprox_select_mso(msoBaudrate);

  for (retry=0; retry<3; retry++)
  {
    watchdog_update();

    /* Reset the MSO */
    msoRequestCounter = 0;
    watchdog_update();

    /* Establish dialog at 9600 bps */   
    serial_0_init(msoBaudrate);
    watchdog_update();

    /* Wait for the MSO to be ready */
    sleep_ms(200);
    watchdog_update();
    
    if (MSO_PP_Send(0x61, 0, send_get, sizeof(send_get)) == ILV_OK)
    {
      if (MSO_PP_Recv(NULL, NULL, NULL, NULL, 500) == ILV_OK)
      {
        /* Great ! */
        sprox_external.active.p.mso = TRUE;
        return ILV_OK;
      }
    }
  
    serial_0_break(100);    
  }

  return ILVERR_ERROR;
}

void MSO_Close(void)
{
  sprox_external.active.p.mso = FALSE;
  if (!sprox_external.active.u)
    PERIPHS_OFF();
  serial_0_break(100);
}

/*************************************************************************
 * Example Table Driven CRC16 Routine using 4-bit message chunks
 *
 * By Ashley Roll
 * Digital Nemesis Pty Ltd
 * www.digitalnemesis.com
 *
 * The following is an example of implementing a restricted size CRC16
 * table lookup. No optimisation as been done so the code is clear and
 * easy to understand.
 *
 * Test Vector: "123456789" (character string, no quotes)
 * Generated CRC: 0x29B1
 * 
 *************************************************************************/

/* 
 * CRC16 Lookup tables (High and Low Byte) for 4 bits per iteration. 
 */
BYTE    CRC16_LookupHigh[16] = {
  0x00, 0x10, 0x20, 0x30, 0x40, 0x50, 0x60, 0x70,
  0x81, 0x91, 0xA1, 0xB1, 0xC1, 0xD1, 0xE1, 0xF1
};
BYTE    CRC16_LookupLow[16] = {
  0x00, 0x21, 0x42, 0x63, 0x84, 0xA5, 0xC6, 0xE7,
  0x08, 0x29, 0x4A, 0x6B, 0x8C, 0xAD, 0xCE, 0xEF
};

/*
 * Process 4 bits of the message to update the CRC Value.
 *
 * Note that the data must be in the low nibble of val.
 */
void MSO_CRCUpdate4Bits(WORD * crc, BYTE val)
{
  BYTE    t;
  register BYTE CRC16_Low = *crc % 0x0100;
  register BYTE CRC16_High = *crc / 0x0100;

  // Step one, extract the Most significant 4 bits of the CRC register
  t = CRC16_High >> 4;

  // XOR in the Message Data into the extracted bits
  t = t ^ val;

  // Shift the CRC Register left 4 bits
  CRC16_High = (CRC16_High << 4) | (CRC16_Low >> 4);
  CRC16_Low = CRC16_Low << 4;

  // Do the table lookups and XOR the result into the CRC Tables
  CRC16_High = CRC16_High ^ CRC16_LookupHigh[t];
  CRC16_Low = CRC16_Low ^ CRC16_LookupLow[t];

  *crc = CRC16_High * 0x0100 + CRC16_Low;
}

/*
 * Process one Message Byte to update the current CRC Value
 */
void MSO_CRCUpdate(WORD * crc, BYTE val)
{
  MSO_CRCUpdate4Bits(crc, val >> 4);  /* High nibble first */
  MSO_CRCUpdate4Bits(crc, val & 0x0F);  /* Low nibble */
}
