#ifndef _CARDAPPZ_JAYCOS_H_
#define _CARDAPPZ_JAYCOS_H_

#include "cardappz_types.h"
#include "cardappz_errors.h"

/*
 * Corporate ID application
 * ------------------------
 */
#define CORPID_JAYCOS_AID 0x9101
#define CORPID_JAYCOS_FID_IDENTITY 1
#define CORPID_JAYCOS_FID_FINGERPRINTS 2
#define CORPID_JAYCOS_FID_PICTURE 3
#define CORPID_JAYCOS_FID_ACCESSCONTROL 4
#define CORPID_JAYCOS_FID_OPTIONSSIGN 5
#define CORPID_JAYCOS_KEY0_ID 2
#define CORPID_JAYCOS_KEY1_ID 3

/* Read-only mode */
/* -------------- */

/* Select application, authentication with key 1 */
int JaycosSelectCorpid(CARDAPPZ_CTX_ST *ctx);

/* Read different files */
int JaycosReadCorpidIdentity(CARDAPPZ_CTX_ST *ctx, CORPID_IDENTITY_ST *identity);
int JaycosReadCorpidFingerprints(CARDAPPZ_CTX_ST *ctx, unsigned char fingerprints[CORPID_FINGERPRINTS_MAX_SZ]);
int JaycosReadCorpidPicture(CARDAPPZ_CTX_ST *ctx, unsigned char picture[CORPID_PICTURE_MAX_SZ]);
int JaycosReadCorpidAccessControl(CARDAPPZ_CTX_ST *ctx, unsigned char accesscontrol[48]);
int JaycosReadCorpidOptionsSign(CARDAPPZ_CTX_ST *ctx, CORPID_OPTIONSSIGN_ST *optionssign);

/* Personalisation */
/* --------------- */

int JaycosWriteCorpid(CARDAPPZ_CTX_ST *ctx, CORPID_IDENTITY_ST *identity, unsigned char fingerprints[CORPID_FINGERPRINTS_MAX_SZ], unsigned char picture[CORPID_PICTURE_MAX_SZ], unsigned char accesscontrol[48], CORPID_OPTIONSSIGN_ST *optionssign);

/* Formatting */
/* ---------- */
int JaycosFormatCorpid(CARDAPPZ_CTX_ST *ctx);

/*
 * Electronic purse application
 * ----------------------------
 */
#define PURSE_JAYCOS_AID 0x9102
#define PURSE_JAYCOS_FID_BALANCE 1
#define PURSE_JAYCOS_FID_RELOAD 2
#define PURSE_JAYCOS_FID_PURCHASE 3
#define PURSE_JAYCOS_KEY0_ID 4
#define PURSE_JAYCOS_KEY1_ID 5

/* Read and decrement mode */
/* ----------------------- */

/* Select application, authentication with key 1 */
int JaycosSelectPurse(CARDAPPZ_CTX_ST *ctx);

/* Read different files */
int JaycosReadPurseBalance(CARDAPPZ_CTX_ST *ctx, unsigned long *Balance);
int JaycosReadPurseReloadEvent(CARDAPPZ_CTX_ST *ctx, int Index, PURSE_RELOAD_ST *ReloadEntry);
int JaycosReadPursePurchaseEvent(CARDAPPZ_CTX_ST *ctx, int Index, PURSE_PURCHASE_ST *PurchaseEntry);

/* New purchase */
int JaycosDebitPurse(CARDAPPZ_CTX_ST *ctx, PURSE_PURCHASE_ST *PurchaseEntry);

/* Reloading */
/* --------- */

int JaycosCreditPurse(CARDAPPZ_CTX_ST *ctx, PURSE_RELOAD_ST *ReloadEntry);

/* Formatting */
/* ---------- */

int JaycosFormatPurse(CARDAPPZ_CTX_ST *ctx);

/*
 * Public transportation application
 * ---------------------------------
 */
#define TRANSPORT_JAYCOS_AID 0x9103
#define TRANSPORT_JAYCOS_FID_CONTRACTS 1
#define TRANSPORT_JAYCOS_FID_EVENTS 2
#define TRANSPORT_JAYCOS_FID_COUNTER1 3
#define TRANSPORT_JAYCOS_FID_COUNTER2 4
#define TRANSPORT_JAYCOS_FID_COUNTER3 5
#define TRANSPORT_JAYCOS_KEY0_ID 6
#define TRANSPORT_JAYCOS_KEY1_ID 7

/* Read and decrement mode */
/* ----------------------- */

/* Select application, authentication with key 1 */
int JaycosSelectTransport(CARDAPPZ_CTX_ST *ctx);

/* Read different files */
int JaycosReadTransportContract(CARDAPPZ_CTX_ST *ctx, int Index, TRANSPORT_CONTRACT_ST *Contract);
int JaycosReadTransportCounter(CARDAPPZ_CTX_ST *ctx, int Index, unsigned long *Value);
int JaycosReadTransportEvent(CARDAPPZ_CTX_ST *ctx, int Index, TRANSPORT_EVENT_ST *Event);

/* New event + decrement counter if needed */
int JaycosAppendTransportEvent(CARDAPPZ_CTX_ST *ctx, TRANSPORT_EVENT_ST *Event);

/* Contract writing */
/* ---------------- */

int JaycosWriteTransportContract(CARDAPPZ_CTX_ST *ctx, int Index, TRANSPORT_CONTRACT_ST *Contract, unsigned long CounterValue);

/* Formatting */
/* ---------- */

int JaycosFormatTransport(CARDAPPZ_CTX_ST *ctx);

/*
 * Common part
 * -----------
 */
//#define DESFIRE_COMM_PLAIN  0
//#define DESFIRE_COMM_MACED  1
//#define DESFIRE_COMM_CIPHER 3
 
static unsigned char JAYCOS_MASTER_KEY[16] = {0xFA,0x1F,0x8D,0x09,0x17,0xF4,0xD2,0x2F,0xC8,0x71,0xD2,0x6A,0x25,0xD2,0xD0,0xFF};

/*
int DesfireAuthenticate(CARDAPPZ_CTX_ST *ctx, unsigned char KeyIndex, const unsigned char KeyValue[16]);
int DesfireChangeKey(CARDAPPZ_CTX_ST *ctx, unsigned char KeyIndex, const unsigned char KeyValue[16]);
int DesfireCreateApplication(CARDAPPZ_CTX_ST *ctx, unsigned long Aid, unsigned char KeySettings, unsigned char KeyCount);
int DesfireSelectApplication(CARDAPPZ_CTX_ST *ctx, unsigned long Aid);
int DesfireSelectCard(CARDAPPZ_CTX_ST *ctx);
int DesfireFormatPICC(CARDAPPZ_CTX_ST *ctx);
int DesfireCreateStdDataFile(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned char CommSettings, unsigned short AccessRights, unsigned long FileSize);
int DesfireCreateValueFile(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned char CommSettings, unsigned short AccessRights, signed long LowerLimit, signed long UpperLimit, signed long CurrentValue, unsigned char LimitedCredit);
int DesfireCreateLinearRecordFile(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned char CommSettings, unsigned short AccessRights, unsigned long RecordSize, unsigned long RecordCount);
int DesfireCreateCyclicRecordFile(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned char CommSettings, unsigned short AccessRights, unsigned long RecordSize, unsigned long RecordCount);
int DesfireReadData(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned char CommMode, unsigned long Offset, unsigned long Length, unsigned char Buffer[], unsigned long *ActualLength);
int DesfireWriteData(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned char CommMode, unsigned long Offset, unsigned long Length, const unsigned char Buffer[]);
int DesfireGetValue(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned char CommMode, signed long *Value);
int DesfireCredit(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned char CommMode, signed long Value);
int DesfireDebit(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned char CommMode, signed long Value);
int DesfireWriteRecord(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned char CommMode, unsigned long Offset, unsigned long Length, const unsigned char Buffer[]);
int DesfireReadRecords(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned char CommMode, unsigned long Offset, unsigned long Length, unsigned char Buffer[], unsigned long *ActualLength);
int DesfireCommitTransaction(CARDAPPZ_CTX_ST *ctx);
int DesfireAbortTransaction(CARDAPPZ_CTX_ST *ctx);
*/
int JaycosCreateMF(CARDAPPZ_CTX_ST *ctx);
int JaycosCreateDF(CARDAPPZ_CTX_ST *ctx, unsigned short DirectoryIndex);

int JaycosCreateTransparentEF(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned short Size);

int JaycosSelectMF(CARDAPPZ_CTX_ST *ctx);
int JaycosSelectDF(CARDAPPZ_CTX_ST *ctx, unsigned short DirectoryIndex);
int JaycosSelectEF(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex);

int JaycosExternalAuthenticate(CARDAPPZ_CTX_ST *ctx, unsigned char KeyIndex, unsigned char KeyValue[16]);
int JaycosMutualAuthenticate(CARDAPPZ_CTX_ST *ctx, unsigned char KeyIndex, unsigned char KeyValue[16]);
int JaycosChangeKey(CARDAPPZ_CTX_ST *ctx, unsigned char KeyIndex, unsigned char KeyValue[16]);

int JaycosReadBinary(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned short Offset, unsigned short Length, unsigned char Buffer[]);
int JaycosReadBinaryEx(CARDAPPZ_CTX_ST *ctx, unsigned short Offset, unsigned char Length, unsigned char Buffer[]);
int JaycosUpdateBinary(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned short Offset, unsigned short Length, unsigned char Buffer[]);
int JaycosUpdateBinaryEx(CARDAPPZ_CTX_ST *ctx, unsigned short Offset, unsigned char Length, unsigned char Buffer[]);

int JaycosSelectCard(CARDAPPZ_CTX_ST *ctx);
int JaycosBlankCard(CARDAPPZ_CTX_ST *ctx);

#endif
